# AMPL script for the firehouse (facility location) problem
param n_sites >= 1, integer;
param n_district >= 1, integer;

set SITES := 1..n_sites;
set DISTRICTS := 1..n_district;

param budget >= 0;

param build_cost {SITES} >= 0;
param service_cost {SITES} >= 0;
param distance {DISTRICTS, SITES} >= 0;
# No fractional population ;-)
param population {DISTRICTS} >= 0, integer; 

# Our "Big-M" (it is not a capital letter because of the naming
# conventions)
param bigm >= 0;

# Decision variables
# Build site or not
var Y {j in SITES} binary;

# Assignment between sites and districts
var X {i in DISTRICTS, j in SITES} binary;

# Maximal distance
var D >= 0;

# Building and service costs
var C >= 0;

# People serviced by a site
var S {j in SITES} >= 0;

# Objective function (minimize maximal distance)
minimize Obj: D;

# Each district is assigned to a site
subject to Assign {i in DISTRICTS}:
	sum {j in SITES} X[i,j] = 1;

# When a site is used, it has to be build
# Formulated as Big-M.
subject to BuildNeeded {j in SITES}:
	sum {i in DISTRICTS} X[i,j] <= bigm * Y[j];

# Definition of the maximum distance
subject to MaxDistance {i in DISTRICTS}:
	D >= sum{j in SITES} distance[i,j]*X[i,j];

# Costs for building the facilties and servicing the population.
subject to Costs:
	sum {j in SITES} (build_cost[j] * Y[j] +
	              service_cost[j] * S[j]) = C;

# Budget constraint
subject to Budget:
	C <= budget;

# Definiton of people served by a certain site
subject to People {j in SITES}:
	S[j] = sum{i in DISTRICTS} (population[i] * X[i,j]);


end;
